/**
* DevExtreme (ui/overlay.d.ts)
* Version: 20.1.4 (build 20169-1806)
* Build date: Wed Jun 17 2020
*
* Copyright (c) 2012 - 2020 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    animationConfig
} from '../animation/fx';

import '../jquery_augmentation';

import {
    dxElement
} from '../core/element';

import {
    template
} from '../core/templates/template';

import {
    event
} from '../events/index';

import Widget, {
    WidgetOptions
} from './widget/ui.widget';

/** Warning! This type is used for internal purposes. Do not import it directly. */
export interface dxOverlayOptions<T = dxOverlay> extends WidgetOptions<T> {
   /** Configures widget visibility animations. This object contains two fields: show and hide. */
    animation?: dxOverlayAnimation;
   /** Specifies whether to close the widget if a user clicks outside it. */
    closeOnOutsideClick?: boolean | ((event: event) => boolean);
   /** Specifies a custom template for the widget content. */
    contentTemplate?: template | ((contentElement: dxElement) => string | Element | JQuery);
   /** Specifies whether to render the widget's content when it is displayed. If false, the content is rendered immediately. */
    deferRendering?: boolean;
   /** Specifies whether or not an end-user can drag the widget. */
    dragEnabled?: boolean;
   /** Specifies the widget's height in pixels. */
    height?: number | string | (() => number | string);
   /** Specifies the maximum height the widget can reach while resizing. */
    maxHeight?: number | string | (() => number | string);
   /** Specifies the maximum width the widget can reach while resizing. */
    maxWidth?: number | string | (() => number | string);
   /** Specifies the minimum height the widget can reach while resizing. */
    minHeight?: number | string | (() => number | string);
   /** Specifies the minimum width the widget can reach while resizing. */
    minWidth?: number | string | (() => number | string);
   /** A function that is executed after the widget is hidden. */
    onHidden?: ((e: { component?: T, element?: dxElement, model?: any }) => any);
   /** A function that is executed before the widget is hidden. */
    onHiding?: ((e: { component?: T, element?: dxElement, model?: any, cancel?: boolean }) => any);
   /** A function that is executed before the widget is displayed. */
    onShowing?: ((e: { component?: T, element?: dxElement, model?: any }) => any);
   /** A function that is executed after the widget is displayed. */
    onShown?: ((e: { component?: T, element?: dxElement, model?: any }) => any);
   /** Positions the widget. */
    position?: any;
   /** Specifies whether to shade the background when the widget is active. */
    shading?: boolean;
   /** Specifies the shading color. Applies only if shading is enabled. */
    shadingColor?: string;
   /** A Boolean value specifying whether or not the widget is visible. */
    visible?: boolean;
   /** Specifies the widget's width in pixels. */
    width?: number | string | (() => number | string);
}
/** Warning! This type is used for internal purposes. Do not import it directly. */
export interface dxOverlayAnimation {
   /** An object that defines the animation options used when the widget is being hidden. */
    hide?: animationConfig;
   /** An object that defines the animation options used when the widget is being shown. */
    show?: animationConfig;
}
/** Warning! This type is used for internal purposes. Do not import it directly. *//** The Overlay widget represents a window overlaying the current view. It displays data located within the HTML element representing the widget. */
export default class dxOverlay extends Widget {
    constructor(element: Element, options?: dxOverlayOptions)
    constructor(element: JQuery, options?: dxOverlayOptions)
   /** Gets the widget's content. */
    content(): dxElement;
   /** Hides the widget. */
    hide(): Promise<boolean> & JQueryPromise<boolean>;
   /** Recalculates the widget's size and position without rerendering. */
    repaint(): void;
   /** Shows the widget. */
    show(): Promise<boolean> & JQueryPromise<boolean>;
   /** Shows or hides the widget depending on the argument. */
    toggle(showing: boolean): Promise<boolean> & JQueryPromise<boolean>;
}
/** Specifies the base z-index for all overlay widgets. */
export function baseZIndex(zIndex: number): void;

/** Warning! This type is used for internal purposes. Do not import it directly. */
export type Options = dxOverlayOptions;

/** @deprecated use Options instead */
/** Warning! This type is used for internal purposes. Do not import it directly. */
export type IOptions = dxOverlayOptions;