/**
* DevExtreme (viz/linear_gauge.d.ts)
* Version: 20.1.4 (build 20169-1806)
* Build date: Wed Jun 17 2020
*
* Copyright (c) 2012 - 2020 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    dxBaseGauge,
    BaseGaugeOptions,
    BaseGaugeRangeContainer,
    BaseGaugeScale,
    BaseGaugeScaleLabel,
    GaugeIndicator
} from './gauges/base_gauge';

/** Warning! This type is used for internal purposes. Do not import it directly. */
export interface dxLinearGaugeOptions extends BaseGaugeOptions<dxLinearGauge> {
   /** Specifies the options required to set the geometry of the LinearGauge widget. */
    geometry?: { orientation?: 'horizontal' | 'vertical' };
   /** Specifies gauge range container options. */
    rangeContainer?: dxLinearGaugeRangeContainer;
   /** Specifies the gauge's scale options. */
    scale?: dxLinearGaugeScale;
   /** Specifies the appearance options of subvalue indicators. */
    subvalueIndicator?: GaugeIndicator;
   /** Specifies the appearance options of the value indicator. */
    valueIndicator?: GaugeIndicator;
}
/** Warning! This type is used for internal purposes. Do not import it directly. */
export interface dxLinearGaugeRangeContainer extends BaseGaugeRangeContainer {
   /** Specifies the orientation of the range container. Applies only if the geometry.orientation option is "vertical". */
    horizontalOrientation?: 'center' | 'left' | 'right';
   /** Specifies the orientation of the range container. Applies only if the geometry.orientation option is "horizontal". */
    verticalOrientation?: 'bottom' | 'center' | 'top';
   /** Specifies the width of the range container's start and end boundaries in the LinearGauge widget. */
    width?: { end?: number, start?: number } | number;
}
/** Warning! This type is used for internal purposes. Do not import it directly. */
export interface dxLinearGaugeScale extends BaseGaugeScale {
   /** Specifies the orientation of scale ticks. Applies only if the geometry.orientation option is "vertical". */
    horizontalOrientation?: 'center' | 'left' | 'right';
   /** Specifies common options for scale labels. */
    label?: dxLinearGaugeScaleLabel;
   /** Specifies the minimum distance between two neighboring major ticks in pixels. */
    scaleDivisionFactor?: number;
   /** Specifies the orientation of scale ticks. Applies only if the geometry.orientation option is "horizontal". */
    verticalOrientation?: 'bottom' | 'center' | 'top';
}
/** Warning! This type is used for internal purposes. Do not import it directly. */
export interface dxLinearGaugeScaleLabel extends BaseGaugeScaleLabel {
   /** Specifies the spacing between scale labels and ticks. */
    indentFromTick?: number;
}/** The LinearGauge is a widget that indicates values on a linear numeric scale. */
export default class dxLinearGauge extends dxBaseGauge {
    constructor(element: Element, options?: dxLinearGaugeOptions)
    constructor(element: JQuery, options?: dxLinearGaugeOptions)
}

declare global {
interface JQuery {
    dxLinearGauge(): JQuery;
    dxLinearGauge(options: "instance"): dxLinearGauge;
    dxLinearGauge(options: string): any;
    dxLinearGauge(options: string, ...params: any[]): any;
    dxLinearGauge(options: dxLinearGaugeOptions): JQuery;
}
}
/** Warning! This type is used for internal purposes. Do not import it directly. */
export type Options = dxLinearGaugeOptions;

/** @deprecated use Options instead */
/** Warning! This type is used for internal purposes. Do not import it directly. */
export type IOptions = dxLinearGaugeOptions;